<?php
/**
 * Created by PhpStorm.
 * User: TREULIAUX
 * Date: 12/12/2016
 * Time: 18:10
 */

/**
 * @file
 * Contains \Drupal\media_upload\Form\MediaUploadConfigurationForm.
 */

namespace Drupal\media_upload\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\media_entity\Entity\MediaBundle;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;

/**
 * Contribute form.
 */
class MediaUploadConfigurationForm extends ConfigFormBase {

  const COMPLETE_FILE_NAME = 0;

  const FILE_NAME = 1;

  const TAG_NAME = 2;

  const EXT_NAME = 3;

  private $_mediaBundle = [
    'audio'    => FALSE,
    'video'    => FALSE,
    'document' => FALSE,
    'image'    => FALSE,
  ];

  public function __construct(
    \Drupal\Core\Config\ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($config_factory);

    //        $moduleHandler = \Drupal::service('module_handler');
    //
    //        foreach ($this->_mediaBundle as $item => $value) {
    //            if ($moduleHandler->moduleExists("media_entity_$item")) {
    //                $this->_mediaBundle[$item] = true;
    //            }
    //        }
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {

    return 'mediaupload_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
    Request $request = NULL
  ) {
    $this->checkMediaBundles();

    $config = $this->config('media_upload.settings');
    $form['bundles'] = [
      '#type'          => 'checkboxes',
      '#title'         => $this->t('Accepted medias'),
      '#options'       => [
        'video'    => $this->t('Videos'),
        'image'    => $this->t('Images'),
        'document' => $this->t('Documents'),
        'audio'    => $this->t('Audios'),
      ],
      '#default_value' => $config->get('bundles'),
    ];
    //        foreach ($this->_mediaBundle as $item => $value) {
    //            if (!$value) {
    //                $form['bundles'][$item]['#disabled'] = true;
    //                drupal_set_message($this->t(ucfirst($item)." - This option requires the following module : media_entity_$item"), 'status', false);
    //            }
    //        }
    $form['total_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max total size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('total_size'),
    ];
    $form['single_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max single file size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('single_size'),
    ];

    $form['videos'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Videos parameters'),
      '#states' => [
        'invisible' => [
          ':input[name="bundles[video]"]' => ['checked' => FALSE],
        ],
        'disabled'  => [
          ':input[name="bundles[video]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['videos']['video_ext'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed videos extensions'),
      '#description'   => $this->t('Accepted extensions for videos media; comma-separated.') . ' Ex: mp4, avi, ...',
      '#default_value' => $config->get('video_ext'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[video]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[video]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['videos']['video_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max video size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('video_size'),
    ];

    $form['images'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Images parameters'),
      '#states' => [
        'invisible' => [
          ':input[name="bundles[image]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['images']['image_ext'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed images extensions'),
      '#description'   => $this->t('Accepted extensions for videos media; comma-separated.') . ' Ex: jpg, png, ...',
      '#default_value' => $config->get('image_ext'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[image]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[image]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['images']['image_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max image size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('image_size'),
    ];

    $form['documents'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Documents parameters'),
      '#states' => [
        'invisible' => [
          ':input[name="bundles[document]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['documents']['document_ext'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed documents extensions'),
      '#description'   => $this->t('Accepted extensions for videos media; comma-separated.') . ' Ex: docx, pdf, ...',
      '#default_value' => $config->get('document_ext'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[document]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[document]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['documents']['document_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max document size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('document_size'),
    ];

    $form['audios'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Audios parameters'),
      '#states' => [
        'invisible' => [
          ':input[name="bundles[audio]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['audios']['audio_ext'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed audios extensions'),
      '#description'   => $this->t('Accepted extensions for videos media; comma-separated.') . ' Ex: docx, pdf, ...',
      '#default_value' => $config->get('audio_ext'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[audio]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[audio]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['audios']['audio_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max audio size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('audio_size'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $this->checkExtensions('video_ext', $form_state);
    $this->checkExtensions('image_ext', $form_state);
    $this->checkExtensions('document_ext', $form_state);
    $this->checkExtensions('audio_ext', $form_state);
    $this->isInt('total_size', $form_state);
    $this->isInt('single_size', $form_state);
    $this->isInt('video_size', $form_state);
    $this->isInt('image_size', $form_state);
    $this->isInt('document_size', $form_state);
    $this->isInt('audio_size', $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $config = $this->config('media_upload.settings');

    $config->set('bundles', $values['bundles']);
    $config->set('total_size', $values['total_size']);
    $config->set('single_size', $values['single_size']);

    if (!empty($values['bundles']['video'])) {
      $config->set('video_ext', preg_replace('/\s+/', '', $values['video_ext']));
      $config->set('video_size', $values['video_size']);
    }

    if (!empty($values['bundles']['image'])) {
      $config->set('image_ext', preg_replace('/\s+/', '', $values['image_ext']));
      $config->set('image_size', $values['image_size']);
    }

    if (!empty($values['bundles']['document'])) {
      $config->set('document_ext', preg_replace('/\s+/', '', $values['document_ext']));
      $config->set('document_size', $values['document_size']);
    }

    if (!empty($values['bundles']['audio'])) {
      $config->set('audio_ext', preg_replace('/\s+/', '', $values['audio_ext']));
      $config->set('audio_size', $values['audio_size']);
    }

    $config->save();
    drupal_set_message('Configuration done.');
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {

    return ['media_upload.settings'];
  }

  private function checkExtensions(
    $fieldName,
    FormStateInterface $form_state
  ) {
    $extensions = $form_state->getValue($fieldName);
    $extensions = preg_replace('/\s+/', '', $extensions);
    if (!preg_match('/(^$)|(^([a-z0-9]+)(,\s*[a-z0-9]+)*$)/', $extensions)) {
      $form_state->setErrorByName($fieldName, $this->t('The extensions values must be comma-separated'));
    }
  }

  private function isInt($fieldName, FormStateInterface $form_state) {
    $val = $form_state->getValue($fieldName);
    if (empty($val)) {
      $val = 0;
    }
    if (!is_numeric($val)) {
      $form_state->setErrorByName($fieldName, $this->t('The size must be a positive interger value'));
    }
    else {
      if (intval($val) < 0) {
        $form_state->setErrorByName($fieldName, $this->t('The size must be a positive interger value'));
      }
    }
  }

  private function checkMediaBundles() {
    foreach ($this->config('media_upload.settings')
                  ->get('bundles') as $item => $value) {
      if ($value) {
        if (empty(\Drupal::entityManager()
                         ->getStorage('media_bundle')
                         ->loadByProperties(['type' => $item]))
        ) {
          drupal_set_message($this->t("You don't have media bundle of the $item type. You should <a href=':link'>create one</a>", [
            ':link' => Url::fromRoute('entity.media_bundle.add_form')
                          ->toString(),
          ]), 'warning');
          $form['submit']['#disabled'] = TRUE;
        }
      }
    }
  }
}
